library crud;

import 'package:flutter/foundation.dart';
import 'package:restified/http.dart';
import 'package:rxdart/rxdart.dart';

class GetBloc<T> {
  final String endpoint;
  final Serializable<T> model;

  final BehaviorSubject<List<T>> _all = BehaviorSubject<List<T>>();
  final BehaviorSubject<T> _single = BehaviorSubject<T>();
  final BehaviorSubject<bool> _loading = BehaviorSubject<bool>();

  GetBloc({this.endpoint, this.model}) {
    _loading.sink.add(true);
  }

  getAll() async {
    _loading.sink.add(true);
    List<T> response = await Http().getAll('$endpoint', model);
    _all.sink.add(response);
    _loading.sink.add(false);
  }

  getById(int id) async {
    _loading.sink.add(true);
    T m = await Http().getSingle('$endpoint/$id', model);
    _single.sink.add(m);
    _loading.sink.add(false);
  }

  dispose() {
    _all.close();
    _single.close();
    _loading.close();
  }

  BehaviorSubject<List<T>> get all => _all;

  BehaviorSubject<T> get single => _single;

  BehaviorSubject<bool> get loading => _loading;
}

class ManipulationBloc<T> {
  final String endpoint;
  final int id;

  final BehaviorSubject<UpdateResponse> _updateResponse =
      BehaviorSubject<UpdateResponse>();
  final BehaviorSubject<InsertResponse> _insertResponse =
      BehaviorSubject<InsertResponse>();
  final BehaviorSubject<bool> _submitting = BehaviorSubject<bool>();

  GetBloc<T> _getBloc;

  ManipulationBloc(Serializable<T> model,
      {@required this.endpoint, @required this.id}) {
    if (id != null) {
      _getBloc = GetBloc(endpoint: endpoint, model: model);
      _getBloc.getById(id);
    }
    _submitting.sink.add(false);
  }

  dispose() {
    _updateResponse.close();
    _insertResponse.close();
    _submitting.close();
  }

  update(int id, Map model) async {
    _submitting.sink.add(true);
    UpdateResponse resp = await Http().put('$endpoint/$id', model);
    _updateResponse.sink.add(resp);
    _submitting.sink.add(false);
  }

  insert(Map model) async {
    _submitting.sink.add(true);
    InsertResponse resp = await Http().post('$endpoint', model);
    _insertResponse.sink.add(resp);
    _submitting.sink.add(false);
  }

  BehaviorSubject<T> get model => _getBloc.single;

  BehaviorSubject<bool> get loading => _getBloc.loading;

  BehaviorSubject<UpdateResponse> get updateResponse => _updateResponse;

  BehaviorSubject<InsertResponse> get insertResponse => _insertResponse;

  BehaviorSubject<bool> get submitting => _submitting;
}
